/***************************************************************************
 *		  Copyright (C) 1994  Charles P. Peterson                  *
 *	     4007 Enchanted Sun, San Antonio, Texas 78244-1254             *
 *              Email: Charles_P_Peterson@fcircus.sat.tx.us                *
 *                                                                         *
 *		  This is free software with NO WARRANTY.                  *
 *	      See gfft.c, or run program itself, for details.              *
 *		      Support is available for a fee.                      *
 ***************************************************************************
 *
 * Program:     gfft--General FFT analysis
 * File:        commands.c
 * Purpose:     define command strings
 * Author:      Charles Peterson (CPP)
 * History:     28-May-1993 CPP; Created.
 *              9-Aug-94 CPP (1.10); Minor semantic changes and Hidden3D
 *
 */

#include "gfft.h"

char *default_command (char *arguments)
{
    error_message (UNSUPPORTED_COMMAND);
    RAISE_ERROR (NOTHING_SPECIAL); /* longjmp outa here! */
    return arguments;
}

char *default_argument (char *arguments)
{
    error_message (NO_SUCH_ARGUMENT);
    RAISE_ERROR (NOTHING_SPECIAL); /* longjmp outa here! */
    return arguments;
}

Name_Info_St Gfft_Command[] = {

  {"?", "?", help_message},
  {"About", "AB", about_message},
  {"Amplitude", "AM", set_amplitude},
  {"Append", "AP", set_append},
  {"Banner", "BA", banner_message},
  {"Bins", "BIN", set_bins},
  {"Bits", "BIT", set_bits},
  {"74dB-Blackman-Harris", "74D", set_blackman_harris_74db},
  {"92dB-Blackman-Harris", "92D", set_blackman_harris_92db},
  {"Calibration", "CAL", set_calibration},
  {"Channel", "CH", set_channel},
  {"CombinePlots", "COM", set_combine_plots},
  {"Copying", "COP", copying_message},
  {"CutCombinedPlot", "CU", cut_combined_plot},
  {"dB", "DB", set_db},
  {"dBCalibration", "DBC", set_db_calibration},
  {"Exit", "E", quit},
  {"FFT-Complex", "FF", set_fft},
  {"Frames", "FR", set_frames},
  {"Go", "G", ok},
  {"Hamming", "HAM", set_hamming},
  {"Hann", "HAN", set_hann},
  {"Help", "HE", help_message},
  {"Hidden3D", "HID", set_hidden_3d},
  {"HighFrequency", "HIGHF", set_high_frequency},
  {"HighY","HIGHY", set_high_y},
  {"IgnoreFormat", "IG", set_ignore_format},
/*  {"Input", "INP", default_command},   Not supported yet  */
  {"Interleave", "INT", set_interleave},
/*  {"Inverse", "INV", default_command},  Not supported yet */
  {"LogX", "LOGX", set_logx},
  {"LogY", "LOGY", set_logy},
  {"LowFrequency", "LOWF", set_low_frequency},
  {"LowY","LOWY", set_low_y},
  {"Mean", "ME", set_mean},
  {"Multiply", "MU", set_multiply},
  {"NoCalibration", "NOCA", set_no_calibrations},
  {"NoCombinePlots", "NOCO", set_no_combine_plots},
  {"NodB", "NOD", set_no_db},
  {"NoHidden3D", "NOH", set_no_hidden_3d},
  {"NoIgnoreFormat", "NOIG", set_no_ignore_format},
  {"NoInterleave", "NOINT", set_no_interleave},
  {"NoLogX", "NOLOGX", set_no_logx},
  {"NoLogY", "NOLOGY", set_no_logy},
  {"NoNumerical", "NON", set_no_numerical},
  {"NoOneShotOnly", "NOON", set_no_one_shot_only},
  {"NoOverlap", "NOOV", set_no_overlap},
  {"NoPad", "NOPAD", set_no_pad},
  {"NoParseval", "NOPAR", set_no_parseval},
  {"NoPink", "NOPI", set_no_pink},
  {"NoPlot", "NOPL", set_no_plot},
  {"NoPSDensity", "NOPS", set_no_psdensity},
  {"NoQuantize","NOQ", set_no_quantization},
  {"NoRepeatOnly", "NOR", set_no_repeat_only},
  {"NoSaveMemory", "NOSA", set_no_save_memory},
  {"NoSquaredSmoothing", "NOSQ", set_no_squared_smoothing},
  {"NoTime3D", "NOT", set_no_time_3d},
  {"NoWarranty", "NOW", no_warranty_message},
  {"Numerical", "NU", set_numerical},
  {"Octave", "OC", set_octave},
  {"OK", "OK", ok},  /* batch_command depends on OK not Ok !!! */
  {"Open", "OP", set_append},
  {"OneShotOnly", "ON", set_one_shot_only},
/*  {"Output", "OU", set_output},  not yet supported */
  {"Overlap", "OV", set_overlap},
  {"Pad", "PAD", set_pad},
  {"Parseval", "PARS", set_parseval},
  {"Parzen", "PARZ", set_parzen},
  {"Pink", "PI", set_pink},
  {"Plot", "PL", set_plot},
  {"PlotOutput", "PLOTO", set_plot_output},
  {"Power", "PO", set_power},
  {"PSDensity", "PS", set_psdensity},
  {"Quantize","QUA", set_quantization},
  {"Quit", "Q", quit},
  {"Rate", "RA", set_rate},
  {"Read", "REA", set_read},
  {"Rectangle", "REC", set_rectangle},
  {"ReOutput", "REO", re_output},
  {"RepeatOnly", "REPE", set_repeat_only},
  {"RePlot", "REPL", re_plot},
  {"RotX", "ROTX", set_rotation_x},
  {"RotZ", "ROTZ", set_rotation_z},
  {"SaveMemory", "SA", set_save_memory},
  {"Set", "SE", set},
  {"ShowSettings", "SH", settings_message},
  {"Signed", "SI", set_signed},
  {"SmoothingSegments", "SM", set_smoothing_segments},
  {"SquaredSmoothing", "SQ", set_squared_smoothing},
  {"StartByte", "STARTB", set_start_byte},
  {"StartFrame", "STARTF", set_start_frame},
  {"Sum", "SU", set_sum},
  {"Terminal", "TE", set_terminal},
  {"Time3D", "TIME3", set_time_3d},
  {"TimeOffset", "TIMEOF", set_time_offset},
  {"TimeOverlap", "TIMEOV", set_time_overlap},
  {"TimeSegments", "TIMESEGM", set_time_segments},
  {"TimeSegSize", "TIMESEGS", set_time_seg_size},
  {"Triangle", "TR", set_triangle},
  {"UnSigned", "U", set_unsigned},
  {"Welch", "WE", set_welch},

#ifdef AMIGA
  {"Workbench", "WO", workbench_command},
#endif

  {"Write", "WR", set_write},
  { "", "", default_command}
};

