/***************************************************************************
 *                Copyright (C) 1994  Charles P. Peterson                  *
 *           4007 Enchanted Sun, San Antonio, Texas 78244-1254             *
 *              Email: Charles_P_Peterson@fcircus.sat.tx.us                *
 *                                                                         *
 *                This is free software with NO WARRANTY.                  *
 *            See gfft.c, or run program itself, for details.              *
 *                    Support is available for a fee.                      *
 ***************************************************************************
 *
 * Program:     gfft--General FFT analysis
 * File:        messages.h
 * Purpose:     message strings
 * Author:      Charles Peterson (CPP)
 * History:     6-August-1993 CPP; Created.
 *              6-July-1994 CPP (0.73); Changed 'About' message
 *              10-July-1994 CPP (1.00); First major release version
 *              27-July-1994 CPP (1.03); added 'f' FPU designation
 * Comments:    In case an alternate language is desired for messages,
 *              only this include should have to be modified.
 *
 *              Only spaces should be used for whitespace (no tabs).
 *              Linefeeds are normally provided by output functions.
 *              
 *                                  *** IMPORTANT ***
 *              ** This is only intended to be included by message.c **
 *              ** The purpose is to locate all strings in one file. **
 */

/*
 * the first two definitions are modified for BETA version
 */

const char NameAuthor[] = "GFFT Version CPP-";
/* const char NameAuthor[] = "GFFT Version BETA-"; */

static const char version[] = {'$','V','E','R',':',' ','C','P','P','/',

/*static const char version[] = {'$','V','E','R',':',' ','B','E','T','A','/', */

'G','F','F','T',' '}; /* no NULL here; terminated later */

/* The version number is variable length x.y */
const char VersionData[] = {'1','.','1','2'};  /* no NULL here */

#ifdef _M68881
static const char fast_char = {'f'};  /* no NULL here */
#endif

static const char filler_space[] = {' '}; /* no NULL here */
#ifdef AMIGA
const char CompileDate[] = __AMIGADATE__;  /* NULL terminated */
#else
const char CompileDate[] = "(x.x.94)";  /* NULL terminated */
#endif

const char Copyright[] = {'C','o','p','y','r','i','g','h','t',' '};
  /* no NULL here; terminated later */

#ifdef AMIGA
const char CopyrightSymbol[] = {(unsigned char) 169};
#else
const char CopyrightSymbol[] = {'(','c',')'};
#endif

static const char copyright_conclusion[] = " 1994 Charles P. Peterson";
  /* Null terminated here */

char *HelpPathList[] = HELP_PATH_LIST;  /* From Xdef.h where X is system */

/*
 * Author strings is specially formatted for output by author() and
 *   about_message ()...
 * Blocks of up to 6 text strings, separated by one line containing one
 *   blank.
 */
char *Author_Strings[] = {
" ",
"Author:   Charles P. Peterson",
"Email:    Charles_P_Peterson@fcircus.sat.tx.us",
"Address:  4007 Enchanted Sun, San Antonio, TX 78244-1254 USA",
" ",
"Cash donations in ANY amount will help support this product.",
"Equipment loans and/or gifts are also appreciated.",
"Specific enhancements and support are also available for a fee.",
" ",
"Service prices (payable in US dollars only) through December 31, 1994:",
"(Prices for delivery outside U.S. shown in parentheses.)",
"Deluxe GFFT 1.x distribution...............................$15 ($20).",
"Expanded Deluxe 1.x distribution...........................$25 ($40).", 
"Early subscription to GFFT 2.0 (projected release: 1Q95)...$30 ($40).",
"For additional details and options (including how you can make a",
"\'Contingent Donation\' or \'Share the Wealth\') see the README file.",
" ",
NULL};

static char *banner_strings[] = {

/* IMPORTANT!
 * Do not modify the following in any way that would obscure the
 * copyright notice and information about the ABSOLUTELY NO WARRANTY and
 * the (NO) WARRANTY and COPYING commands.  To do so might be a violation
 * of the GNU General Public License under which this software is
 * distributed.
 */

"\n",
"         There is ABSOLUTELY NO WARRANTY; enter NOWARRANTY for details",
"         You may give out copies of GFFT; enter COPYING for conditions",
"             Please support this product--order services or donate",
"         For further information and the author\'s address, enter ABOUT",
"                              Enter HELP for help",
" ",
NULL};

static char *help_prolog_strings[] = {
"The following commands are available in this program:",
" ",
NULL};

static char *help_epilog_strings[] = {
" ",
"Any command may be abbreviated to its shortest unambiguous string.",
"Commands are not case sensitive in this program.",
" ",
"For help on any particular command, enter help <command name>.",
"For list of additional help topics, enter help topics.",
"For an introduction to GFFT, enter help introduction.",
" ",
"Hint: to begin processing after desired parameters have been modified,",
"enter the OK command.  For example:",
" ",
"    gfft>read mysample",
"    gfft>bins 1024",
"    gfft>ok",
" ",
NULL};

static char *question_message_strings[] = {
"Batch Usage: GFFT [READ filename WRITE filename [options]+ ]",
"If no arguments are given, interactive mode is entered.",
"For a complete list of options/commands, enter GFFT HELP.",
NULL};

/*
 * Workbench strings
 */
char *Please_Select_Read_File_S =
"Please select a Sample File (above)...";

char *Set_Parameters_Then_Ok_S =
 "Set parameters, then press OK...";

char *Attempting_to_Perform_FFT_S =
"Performing FFT analysis...";

char *Set_Sampling_Rate_S =
"Now set sampling rate (file is unformatted)...";

char *Verify_Sampling_Rate_S = 
"Verify sampling rate, then set parameters...";

char *Got_It_S = 
"Got it.";

char *Read_Requester_Title =
"Select READ File for GFFT";

char *Dialog_Window_Title =
"GFFT Dialog Window";

char *Copyrt_Req_Prolog =
"To support GFFT, please donate, subscribe, and/or order services...";

char *Author_Info_Button =
"Press here for information about the author and services...";

char *No_Warranty_Button =
"There is ABSOLUTELY NO WARRANTY.  Press here for details...";

char *Sharing_Rights_Button =
"You may give out copies of GFFT.  Press here for conditions...";

char *Idea_Contributor_Button =
"Many have contributed to the ideas behind this program...";

char *Code_Contributor_Button =
"Code has been contributed by...";

/*
 * Error message strings
 * String must NOT include null!
 * Each should be a one-liner if possible
 * Follow style here, or modify all to different style
 */

#define NO_SUCH_COMMAND_S \
"No such command!"

#define OUT_OF_MEMORY_S \
"Insufficient memory to complete operation!  Try fewer bins."

#define UNSUPPORTED_COMMAND_S \
"Command not currently supported on this system!"

#define UNDOCUMENTED_ERROR_S \
"An error for which no description is available has occurred!"

#define MISSING_ARGUMENT_S \
"Missing argument!"

#define BAD_ARGUMENT_S \
"Invalid argument!"

#define BAD_NAME_S \
"Invalid name!"

#define AMBIGUOUS_COMMAND_S \
"Ambiguous command!"

#define BACKUP_CREATED_S \
"Warning.  Backup file created to prevent overwriting ONCE ONLY."

#define OUTPUT_BACKUP_FAILURE_S \
"Unable to backup existing file!  File not opened for output!"

#define BACKUP_OVERWRITTEN_S \
"Warning.  Previous backup file is now being overwritten by new backup file."

#define CANT_CREATE_OUTPUT_FILE_S \
"Unable to create file for output!  Bad name or protected directory!"

#define CANT_CLEANUP_BATCH_PLOT_S \
"Warning.  Can't cleanup temporary files if plotting from batch mode."

#define CANT_PLOT_FROM_STDOUT_S \
"Unable to plot if data written to stdout!"

#define CANT_RE_OUTPUT_S \
"Can't re-output!  Need to do OK first."

#define CANT_RE_PLOT_S \
"Can't re-plot!  Need to do OK first."

#define CANT_OPEN_INPUT_FILE_S \
"Unable to open input file!  No such file!"

#define SCANNING_FILE_S \
"Scanning input file to determine number of bins..."

#define NO_DATA_PRESENT_S \
"Input file does not contain any data!"

#define PADDING_TAILEND_S \
"Warning.  Padding last section of data with zeros."

#define IGNORING_TAILEND_S \
"Warning.  Ignoring last partial segment of data."

#define INSUFFICIENT_DATA_S \
"Insufficient data to analyze with specified parameters."

#define AMBIGUOUS_ARGUMENT_S \
"Ambiguous argument!"

#define NO_SUCH_ARGUMENT_S \
"No such argument!"

#define NO_NUMERICAL_S \
"Numerical Recipes code must be purchased from Cambridge University Press!"

#define INVALID_NUMBER_S \
"Invalid number!"

#define FILENAME_TOO_LONG_S \
"Filename is too long!"

#define NO_RATE_SPECIFIED_S \
"No sampling rate has been specified!  No analysis can be performed!"

#define NO_READ_FILE_S \
"No valid input file has been specified!"

#define UNSUPPORTED_FORMAT_S \
"This file appears to an unsupported format type!"

#define STARTUP_FILE_ERROR_S \
"Error in .gfft startup file!"

#define CORRUPT_IFF_S \
"The IFF structure of this file appears to be corrupted!"

#define CORRUPT_AVR_S \
"The AVR format of this file appears to be corrupted!"

#define COMPRESSION_NOT_SUPPORTED_S \
"The compression used in this file is not currently supported!"

#define ONESHOT_BUT_NOT_8SVX_S \
"OneshotOnly or RepeatOnly are only supported for IFF 8SVX files!"

#define FORMAT_NOT_OK_S \
"BAD FORMAT!  The input file cannot be processed!"

#define NOT_ENOUGH_FRAMES_S \
"There are not enough frames in input file to be processed as specified!"

#define OCTAVE_NOT_PRESENT_S \
"The specified octave is not present in input file!"

#define CORRUPT_AIFF_S \
"The format of this AIFF file is corrupt or unsupported by this program!"

#define CORRUPT_AIFC_S \
"The format of this AIFC file is corrupt or unsupported by this program!"

#define NO_WRITE_FILE_S \
"No valid output file has been specified!"

#define DB_OF_ZERO_S \
"Cannot compute dB of a zero value!"

#define NO_FILE_REQSTR_S \
"File selector requires AmigaDOS 2.0+!  Enter file name into name box instead."

#define SPEC_CHANNEL_UNAVAIL_S \
"Specified channel is not available!"

#define INVALID_BINS_SPEC_S \
"Invalid bins specification!"

#define INVALID_INTERLEAVE_SPEC_S \
"Invalid interleave specification!"

#define CANT_CREATE_PLOTTER_FILE_S \
"Unable to write command file for plotter in ram!"

#define CANT_CREATE_MORE_COMMAND_FILE_S \
"Unable to write command file for more command in ram!"

#define CANT_CREATE_MESSAGE_FILE_S \
"Unable to write file with message strings!"

#define HELP_MESSAGE_UNAVAILABLE_S \
"No help is available for that topic!"

#define HELP_FILE_NOT_FOUND_S \
"Help file has not been properly installed!"

#define CANT_OPEN_INTUITION_S \
"Unable to open Intuition library!  Where is your ROM anyway?"

#define INVALID_SMOOTHING_SPEC_S \
"Invalid smoothing segment specification!"

#define INVALID_TIMESEG_PARAMETERS_S \
"Invalid Time Segment Parameters!"

#define MISSING_TIMESEG_PARAMETERS_S \
"Incomplete Time Segment Parameters!"

#define INVALID_ROTATION_SPEC_S \
"Invalid X or Z rotation specified!"

#define CHANGED_FORMAT_DEP_VALUE_S \
"Warning.  Changed inaccessible invalid format dependent value(s)."

#define INVALID_CALIBRATION_FILE_S \
"Invalid line in calibration file!  Unable to use calibration file!"

#define MORE_STRING "Enter <RETURN> for more; Enter Q <RETURN> to quit: "
#define NO_MORE_CHAR_1 'Q'
#define NO_MORE_CHAR_2 'q'


/*
 * Message strings for 'settings' command
 */

#define SAMPLING_RATE_S \
"The sampling rate is "  /* Note, number follows in separate printf */

#define NUMERICAL_RECIPES_USED_S \
"The fft code from Numerical Recipes will be used.\n"

#define AMPLITUDE_SPECTRUM_S \
"An amplitude spectrum analysis will be performed.\n"

#define POWER_SPECTRUM_S \
"A power spectrum analysis will be performed.\n"

#define FFT_S \
"An FFT will be performed producing complex output data.\n"

#define MEAN_NORMALIZED_S \
"The spectrum will be normalized to mean values.\n"

#define SUMMED_S \
"The summed spectral bin values will be output.\n"

#define DEFAULT_BINS_S \
"The number of positive frequency bins will be determined by data count.\n"

#define BINS_S \
"%ld positive frequency bins will be used.\n"

#define RECTANGLE_WINDOWS_S \
"Rectangle windows will be applied to input data segments.\n"

#define TRIANGLE_WINDOWS_S \
"Triangle windows will be applied to input data segments.\n"

#define HAMMING_WINDOWS_S \
"Hamming windows will be applied to input data segments.\n"

#define BLACKMAN_HARRIS_74DB_WINDOWS_S \
"Blackman-Harris 74dB windows will be applied to input data segments.\n"

#define BLACKMAN_HARRIS_92DB_WINDOWS_S \
"Blackman-Harris 92dB windows will be applied to input data segments.\n"

#define HANN_WINDOWS_S \
"Hann windows will be applied to input data segments.\n"

#define PARZEN_WINDOWS_S \
"Parzen windows will be applied to input data segments.\n"

#define WELCH_WINDOWS_S \
"Welch windows will be applied to input data segments.\n"

#define OVERLAP_S \
"Overlapping data segments will be used as much as possible.\n"

#define NO_OVERLAP_S \
"Data segments will not overlap.\n"

#define INTERLEAVE_S \
"Data segments will be interleaved.\n"

#define PINK_S \
"Pink weighting (* Frequency) will be applied to the output.\n"

#define MAX_FREQUENCY_S \
"Results for frequencies above %g will not be reported.\n"

#define MIN_FREQUENCY_S \
"Results for frequencies below %g will not be reported.\n"

#define KEYBOARD_INPUT_S \
"Input data will be read from keyboard.\n"

#define INPUT_FILENAME_S \
"Input file is %s\n"

#define STARTBYTE_S \
"%ld bytes will be skipped from start of file.\n"

#define OUTPUT_SCREEN_S \
"Output results will be displayed on screen.\n"

#define OUTPUT_FILENAME_S \
"Output file is %s\n"

/*
 * Other messages from settings.c
 */

#define BINS_SELECTED_S \
"0 or non-negative power of 2 required.  %lu bins selected."

/*
 * Messages from okspectm.c
 */

#define BINS_DETERMINED_S \
"%lu data frames available; Using %lu positive frequency bins."

/*
 * Information messages
 * [moved to gfft.help]
 */

/*
 * The following may not be modified without violating the copyright terms!
 */

static char *no_warranty_strings[] = {
"			    NO WARRANTY",
" ",
"  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY",
"FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN",
"OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES",
"PROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED",
"OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF",
"MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS",
"TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE",
"PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,",
"REPAIR OR CORRECTION.",
" ",
"  12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING",
"WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR",
"REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,",
"INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING",
"OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED",
"TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY",
"YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER",
"PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE",
"POSSIBILITY OF SUCH DAMAGES.",
NULL};

static char *copying_unavailable_strings[] = {
" ",
"Unable to read file named COPYING!",
"The file named COPYING should have been put in an appropriate place during",
"the installation of this program.  The GNU General Public License is",
"described by that file.  Please locate the file COPYING from your original",
"GFFT distribution, and copy it to a suitable directory as described in the",
"INSTALL file for your system.  If you no longer have the files from your",
"original distribution, you may obtain a copy of the GNU General Public",
"License from the Free Software Foundation, Inc., 675 Mass Ave,",
"Cambridge, MA 02139 USA.  Please do not ask them any other questions about",
"this program.  For information about this program, contact the author,",
"whose address is displayed by the ABOUT command.",
" ",
NULL};
